SystemOrganization addCategory: #'Seaside-Flow'!

!WAPackage class methodsFor: '*seaside-flow' stamp: 'lr 1/24/2009 21:25'!
seasideFlow
	^ self new
		name: 'Seaside-Flow';
		description: 'Define control flow as a continuous piece of code, with natural, linear control flow.';
		addDependency: 'Seaside-Component';
		yourself! !

!WAComponent methodsFor: '*seaside-flow' stamp: 'lr 1/24/2009 21:00'!
call: aComponent 
	"Pass control from the receiver to aComponent. The receiver will be temporarily replaced with aComponent. Code can return from here later on by sending #answer: to aComponent."

	^ self wait: [ :cc | self show: aComponent onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:03'!
chooseFrom: aCollection
	"Choose an item from the given aCollection. Answer the selected item."

	^ self wait: [ :cc | self chooseFrom: aCollection onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
chooseFrom: aCollection caption: aString
	"Choose an item from the given aCollection with caption aString. Answer the selected item."
	
	^ self wait: [ :cc | self chooseFrom: aCollection caption: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
confirm: aString
	"Display a yes/no dialog with caption aString. Answer true if the response is yes, false if no."

	^ self wait: [ :cc | self confirm: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:04'!
inform: aString
	"Display a dialog with aString to the user until he clicks the ok button."

	^ self wait: [ :cc | self inform: aString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString
	"Display an input dialog with the question aRequestString. Answer the string the user enters."

	^ self wait: [ :cc | self request: aRequestString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString default: aDefaultString
	"Display an input dialog with the question aRequestString and the default string aDefaultString. Answer the string the user enters."
	
	^ self wait: [ :cc | self request: aRequestString default: aDefaultString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString label: aLabelString
	"Display an input dialog with the question aRequestString and the button label aLabelString. Answer the string the user enters."
	
	^ self wait: [ :cc | self request: aRequestString label: aLabelString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow-dialog' stamp: 'lr 1/24/2009 21:05'!
request: aRequestString label: aLabelString default: aDefaultString
	"Display an input dialog with the question aRequestString, the button label aLabelString and the default string aDefaultString. Answer the string the user enters."

	^ self wait: [ :cc | self request: aRequestString label: aLabelString default: aDefaultString onAnswer: cc ]! !

!WAComponent methodsFor: '*seaside-flow' stamp: 'lr 1/24/2009 21:03'!
wait: aBlock
	"Evaluate aBlock and pass in a continuation that can be evaluated to answer to the place where this very method was called. Note that after evaluating aBlock a re-rendering of the receiver is enfored. Do not use this method, if you don't understand this comment."

	^ WAPartialContinuation
		marker: (WAActionCallback compiledMethodAt: #evaluateWithArgument:)
		do: [ :cc | aBlock value: cc. WARenderNotification signal ]! !

WAComponent subclass: #WATask
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Flow'!

!WATask commentStamp: 'pmm 6/30/2007 19:02' prior: 0!
I am a subclass of WAComponent, specialized for defining workflow.  The difference between a task and a component is the following:

Both of them are reusable, embeddable, callable pieces of user interface. A component has state (instance variables), behavior (it may change its state, and it may also choose to display other components with #call:), and appearance (it renders HTML). A Task has only the first two - it doesn't render any HTML directly, but only through the components it calls. This is useful when what you want to encapsulate/embed/call is purely a process (show this component, then this one, then this one).

The key method for WATask is #go - as soon as a task is displayed, this method will get invoked, and will presumably #call: other components.

In terms of implementation, you can think of a WATask in the following way: it is a component which renders two things:
- a link whose callback invokes the #go method
- a header that immediately redirects to the URL of that link

Subclasses must not implement #renderContentOn:

An example can be found in WAConvenienceTest.
!

!WATask methodsFor: 'running' stamp: 'jf 11/10/2003 16:13'!
go
	self subclassResponsibility! !

!WATask methodsFor: 'rendering' stamp: 'lr 1/19/2009 18:23'!
updateRoot: aHtmlRoot
	self activeComponent == self
		ifFalse: [ ^ super updateRoot: aHtmlRoot ].
	self requestContext response
		redirectTo: (aHtmlRoot context actionUrl
			withParameter: (aHtmlRoot context callbacks
				registerActionCallback: [ [ self answer: self go ] repeat ]));
		respond! !
